#!/usr/bin/env Rscript 

#--------------------------SCRIPT DESCRIPTION---------------------
#This script calculate density coeficients. Density coeficients are used to calculate weights for dasymetrnic modeling.

library(DBI)
library(RSQLite)

#dasy_parameters.R contain names and paths to input/output files used during calculation. 
source("dasy_parameters.R")

#--------------------------START SCRIPT--------------------------

#connect to database
us_db <- dbConnect(SQLite(), dbname = file.path(us_db_path,us_db_name, fsep = .Platform$file.sep))

#read GISJOIN, cns_id and population from US_CENSUS_2010 table and OUT_AREAS table from db and merge it using merge funcion from r

#read OUT_AREAS table from database to data frame areas
areas=dbReadTable(us_db, "OUT_AREAS")

#select GISJOIN, cns_id, POP from US_CENSUS_2010 table in database and read it to us_pop data frame. US_CENSUS_2010 table is created in r2_grass.R
squery="SELECT cns_id,GISJOIN,POP FROM US_CENSUS_2010"
us_pop=dbGetQuery(us_db, squery)

#merge areas and us_pop data frames and create input_df
#input df contain data about areas and population for each block
input_df=merge(x=us_pop, y=areas, by="GISJOIN", all=TRUE)
input_df=input_df[,-4]
col_names=names(input_df)
col_names[2]<-"cns_id"
colnames(input_df)=col_names

rm(areas, us_pop)
#----------------------------------CALCULATE DENSITY COEFFICIENTS-----------------------------------------
#calculate percent of area of ancillary class in given block 
anc_perc=data.frame(cns_id=input_df$cns_id, state_code=input_df$STATE_CODE, county_code=input_df$COUNTY_CODE, GISJOIN=input_df$GISJOIN, (input_df[,-c(1,2,3,4,5,ncol(input_df))]/input_df[,ncol(input_df)])*100)

list_densities=list()
density_in_anc_class=c()

#calculate for each ancillary class, except uninhabited class
#0.0009 is an area of grid cell, we used data in 30 m resolution 
for (i in ancillary_class[-1]) {
  selector=5+i
  b=input_df[which(input_df$POP>0 & anc_perc[,selector]>thres[i+1]),]
  d=b$POP/(b$area*0.0009)
  list_densities[[i]]=d
  dens=sum(b$POP)/sum((b$area*0.0009))
  density_in_anc_class[i+1]=dens
} 
rm(selector, b,d,dens)

#a0=anc_perc[anc_perc$a0==100]

#calculate density coefincients by divide density in given ancillary class by sum of densities in each class
density_in_anc_class[1]=0
density_coefficients=(density_in_anc_class/sum(density_in_anc_class, na.rm=TRUE))*100  

#save results into rdata
save(list_densities, file=file.path(us_db_path, "list_densities.RData"))
save(density_coefficients, file=file.path(us_db_path, "density_coeficients.RData"))

dbDisconnect(us_db)  
